/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  fff2tgal.c   SS Dec 97                                   */
/*  Linux version.                                           */
/*  Latest update: 1998-06-05                                */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>


int main(argc,argv) 
int argc;
char *argv[];
{

   FILE *infile,*outfile;

   int i,j,k,xsize,ysize,temp;

   double amp,max,amp0,ftemp;

   int xys[3];

   unsigned short ss[10];

   unsigned char dd[4];

   float fff[3];

   if (argc<3) {
      printf("Usage: %s image.fff image.tga ampr\n",argv[0]);
      exit(0);
   } /* if */



   if ((infile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */


   amp0=atof(argv[3]);


   fread(&xys[1],sizeof(int),2,infile);

   xsize=xys[1];
   ysize=xys[2];

   if (amp0 > 0.0) {
      amp=amp0/128.0;
   }
   else {

      max=-1.0e30;

      for (i=0; i<ysize; i++) {
         for (j=0; j<xsize; j++) {
             fread(&fff[1],sizeof(float),1,infile);
             if (fff[1] > max) {
	        max=fff[1];
	     } /* if */
         } /* for j */
      } /* for i */

      amp=255.0/max;

   } /* if */

   ss[1]=0;
   ss[2]=2;
   ss[3]=0;
   ss[4]=0;
   ss[5]=0;
   ss[6]=0;
   ss[7]=xys[1];
   ss[8]=xys[2];
   ss[9]=8216;

   rewind(infile);

   fread(&xys[1],sizeof(int),2,infile);

   outfile = fopen(argv[2],"wb");

   fwrite(&ss[1],sizeof(unsigned short),9,outfile);

   printf("%6d %6d\n",xsize,ysize);
 
   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
          fread(&fff[1],sizeof(float),1,infile);
          ftemp=amp*fff[1];
	  if (ftemp > 255.0) {
	     ftemp=255.0;
	  } /* if */
	  if (ftemp < 0.0) {
	     ftemp=0.0;
	  } /* if */
	  temp=(int)(ftemp+0.5);
          dd[1]=temp;
          dd[2]=temp;
          dd[3]=temp;
          fwrite(&dd[1],sizeof(unsigned char),3,outfile);
      } /* for j */
   } /* for i */


   fclose(outfile);
   fclose(infile);


} /* End  */

