/***************************************************************************e
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*                                                           */
/*   mrebinuf.c    SS June 1995, July 2007                   */
/*                                                           */
/*************************************************************/


#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>


double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


void mshift(a,b,m,n,dx,dy) 
double **a;
double **b;
int m,n;
double dx,dy;
{
   int i,j;
   double f12,f34,f234,f1,f2,f3,f4,p21,pm2,e6;


   e6=1.0/6.0;

   p21=dx*dx-1.0;
   pm2=dx-2.0;

   f1=e6*(-dx)*(dx-1.0)*pm2;
   f2=0.5*p21*pm2;
   f3=0.5*(-dx)*(dx+1.0)*pm2;
   f4=e6*dx*p21;

   f12=f1+f2;
   f34=f3+f4;
   f234=f2+f34;

   for (j=1; j<=n; j++) {
      b[1][j]=f12*a[1][j]+f3*a[2][j]+f4*a[3][j];
      for (i=2; i<=(m-2); i++) {
         b[i][j]=f1*a[i-1][j]+f2*a[i][j]+f3*a[i+1][j]+f4*a[i+2][j];
      } /* for i */
      b[m-1][j]=f1*a[m-2][j]+f2*a[m-1][j]+f34*a[m][j];
      b[m][j]=f1*a[m-1][j]+f234*a[m][j];
   } /* for j */

   for (i=1; i<=m; i++) {
      for (j=1; j<=n; j++) {
         a[i][j]=b[i][j];
      } /* for j */
   } /* for i */

   p21=dy*dy-1.0;
   pm2=dy-2.0;

   f1=e6*(-dy)*(dy-1.0)*pm2;
   f2=0.5*p21*pm2;
   f3=0.5*(-dy)*(dy+1.0)*pm2;
   f4=e6*dy*p21;

   f12=f1+f2;
   f34=f3+f4;
   f234=f2+f34;
  
   for (i=1; i<=m; i++) {
      b[i][1]=f12*a[i][1]+f3*a[i][2]+f4*a[i][3];
      for (j=2; j<=(n-2); j++) {
         b[i][j]=f1*a[i][j-1]+f2*a[i][j]+f3*a[i][j+1]+f4*a[i][j+2];
      } /* for i */
      b[i][n-1]=f1*a[i][n-2]+f2*a[i][n-1]+f34*a[i][n];
      b[i][n]=f1*a[i][n-1]+f234*a[i][n];
   } /* for j */

} /* mshift */


int main(argc,argv)
int argc;
char *argv[];
{
   FILE *infile,*outfile;
   
   int i,j,i2,j2,k,l,bf,w,h,z,xi,yi,i3,j3,w1,h1,w2,h2;
   int xsize,ysize,xsize2,ysize2;

   double x,y,ss;

   int xys[3];
   int xys2[3];

   float fff[3];

   clock_t time1,timediff;

   float **image1,**image2;
   float *dpimage1,*dpimage2;

   double **m1,**m2;
   double *dpm1,*dpm2; 

   if (argc != 4) {
      printf("Usage: %s inimage.fff outimage.fff binfactori\n",argv[0]);
      exit(0);
   } /* if */

   bf=atoi(argv[3]);

   if ((infile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   outfile=fopen(argv[2],"wb");

   fread(&xys[1],sizeof(int),2,infile);

   xsize=xys[1];
   ysize=xys[2];

   w=xsize;
   h=ysize;

   w1=w+1;
   h1=h+1;

   xsize2=bf*xsize;
   ysize2=bf*ysize;

   w2=xsize2;
   h2=ysize2;

   xys2[1]=xsize2;
   xys2[2]=ysize2;

   fwrite(&xys2[1],sizeof(int),2,outfile);


   image1=(float **)calloc(ysize,sizeof(float*));
   dpimage1=(float *)calloc(xsize*ysize,sizeof(float)); 

   image2=(float **)calloc(ysize2,sizeof(float*));
   dpimage2=(float *)calloc(xsize2*ysize2,sizeof(float)); 

   for (i=0; i<ysize; i++) {
      image1[i]=dpimage1;
      dpimage1=dpimage1+xsize;
   } /* for i */

   for (i=0; i<ysize2; i++) {
      image2[i]=dpimage2;
      dpimage2=dpimage2+xsize2;
   } /* for i */

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
         fread(&fff[1],sizeof(float),1,infile);
	 image1[i][j]=fff[1];
      } /* for j */
   } /* for i */  

   m1=(double **)calloc(w1+1,sizeof(double*));
   dpm1=(double *)calloc((w1+1)*(h1+1),sizeof(double)); 

   m2=(double **)calloc(w1+1,sizeof(double*));
   dpm2=(double *)calloc((w1+1)*(h1+1),sizeof(double)); 

   for (i=0; i<=w1; i++) {
      m1[i]=dpm1;
      m2[i]=dpm2;
      dpm1=dpm1+h1+1;
      dpm2=dpm2+h1+1;
   } /* for i */

   ss=1.0/(double)(bf);

   time1=clock();

   for (i2=0; i2<bf; i2++) {
       for (j2=0; j2<bf; j2++) {

          x=(double)(i2)*ss;
          y=(double)(j2)*ss;

          x=1.0-x;
          y=1.0-y;

          for (i=0; i<w; i++){
              for (j=0; j<h; j++) {
                  m1[i+1][j+1]=(double)(image1[j][i]);
               } /* for j */
          } /* for i */

          mshift(m1,m2,w,h,x,y);

          i3=1; 
          for (i=bf-i2-1; i<w2; i=i+bf) {
              j3=1;
              for (j=bf-j2-1; j<h2; j=j+bf) {   
                  image2[j][i]=(float)(m2[i3][j3]);
                  j3++;
             } /* for j */
             i3++;
          } /* for i */

      } /* for j2 */
   } /* for i2 */

   for (i=0; i<ysize2; i++) {
      for (j=0; j<xsize2; j++) {
	 fff[1]=image2[i][j];
	 fwrite(&fff[1],sizeof(float),1,outfile);
      } /* for j */
   } /* for i */  

   timediff=clock()-time1;

   printf("%10.2f\n",(double)(timediff)/timetic);

   fclose(outfile);
   fclose(infile);

} /* End  */
