/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*   lucyf.c  SS July 92, Oct 93, Dec 97                     */
/*                                                           */
/*   Deconvolution using FFT Lucy-Richardson.                */
/*   Single precision.                                       */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


void cstatistics(d,le,stat)
float d[];
int le;
double stat[];
{
  int i,nrofhm,le2;
  double min,max,sum,fwhm,fl,temp,halfmax;

  le2=2*le;;
  fl=(double)(le);
  
  max=-1.0e10;
  min=1.0e10;
  sum=0.0;

  for (i=1; i<=le2; i=i+2) {
     temp=d[i];
     sum=sum+temp;
     if (temp > max) max=temp;
     if (temp < min) min=temp;
  } /* for i */

  halfmax=max/2.0;
  nrofhm=0;

  for (i=1; i<=le2; i=i+2) {
     if (d[i] > halfmax) {
        nrofhm=nrofhm+1;
     }
  } /* for i */

  fwhm=sqrt(4.0*((double)(nrofhm))/pi);

  stat[1]=max;
  stat[2]=min;
  stat[3]=sum/fl;
  stat[4]=fwhm;

} /* cstatistics */

 
void rstatistics(d,le,stat)
float d[];
int le;
double stat[];
{
  int i,nrofhm;
  double min,max,sum,fwhm,fl,temp,halfmax;

  fl=(double)(le);
  
  max=-1.0e10;
  min=1.0e10;
  sum=0.0;

  for (i=1; i<=le; i++) {
     temp=d[i];
     sum=sum+temp;
     if (temp > max) max=temp;
     if (temp < min) min=temp;
  } /* for i */

  halfmax=max/2.0;
  nrofhm=0;

  for (i=1; i<=le; i++) {
     if (d[i] > halfmax) {
        nrofhm=nrofhm+1;
     }
  } /* for i */

  fwhm=sqrt(4.0*((double)(nrofhm))/pi);

  stat[1]=max;
  stat[2]=min;
  stat[3]=sum/fl;
  stat[4]=fwhm;

} /* rstatistics */



void printstat(stat)
double stat[];
{
printf("%12.2f %10.2f %10.2f %10.2f",stat[1],stat[2],stat[3],stat[4]);
} /* printstat */


void prime(n)
int n;
{
  int m,p;

  m=n;
  p=2;

  while (m >= 2) {
     if ((m % p) == 0) {
        m=m/p;
        printf(" %d",p);
     }
     else {
        p++;
     }
  } /* while */

} /* prime */



int main(argc,argv) 
int argc;
char *argv[];

{

   FILE *infile,*outfile,*imfile,*kfile;

   double psfd[9];
   double stat[6];
   double amp[1002];

   double size,invsize,eps,amp0,max,diffsec,sum,ftemp;

   int ss[3];

   int i,j,k,l,width,height,it,itmax,savenr,isave,temp,cc,isign,tt,ndim;
   int kwidth,kheight,kworig,khorig,le,i2,j2,length,maxwh;
 
   int saveit[1002];

   int xys[3];
   int kxys[3];

   char name[80];
   char savename[80];
   char savename2[80];
   char outname[80];
   char kernelname[80];
   char itstr[10];
 
   clock_t time1,timediff;

   float *d0,*d1,*dc,*psf,*w;

   float dd[3];

if (argc < 2) {
   printf("Usage: %s infile\n",argv[0]);
   printf("Parameters: imagename\n"); 
   printf("            savename\n");
   printf("            psfname\n");
   printf("            saveit_1 saveit_2 ... saveit_n flag(<=0)\n");
   exit(0);
} /* if */



if ((infile = fopen(argv[1],"r")) == NULL) {
   printf("File not found.\n");
   exit(0);
} /* if */


fscanf(infile,"%s",name);
fscanf(infile,"%s",savename);
fscanf(infile,"%s",kernelname);

i=1;
do {
   fscanf(infile,"%d",&temp);
   saveit[i]=temp;
   i++;
} while (temp > 0);

savenr=i-2; 
itmax=saveit[savenr]; 


strcat(name,".fff");
strcat(kernelname,".fff");
strcpy(savename2,savename);
strcat(savename2,"_itnr.fff");

if ((imfile = fopen(name,"rb")) == NULL) {
   printf("File not found.\n");
   exit(0);
} /* if */

if ((kfile = fopen(kernelname,"rb")) == NULL) {
   printf("File not found.\n");
   exit(0);
} /* if */


fread(&xys[1],sizeof(int),2,imfile);

width=xys[1];
height=xys[2];

length=width*height;
le=length;

if (width > height) {
   maxwh=width;
}
else {
   maxwh=height;
} /* if */

d0 = (float *)calloc(length+1,sizeof(float));
d1 = (float *)calloc(length+1,sizeof(float));
dc = (float *)calloc(2*length+1,sizeof(float));
psf = (float *)calloc(2*length+1,sizeof(float));
w = (float *)calloc(2*maxwh+1,sizeof(float));


fread(&kxys[1],sizeof(int),2,kfile);
 
kwidth=kxys[1];
kheight=kxys[2];
kworig=(kwidth-1)/2;
khorig=(kheight-1)/2;

ndim=2;
ss[1]=height;
ss[2]=width;


size=(double)(le);
invsize=1.0/size;
eps=1.0;

printf("\n");
printf("Lucy-Richardson deconvolution\n");
printf("=============================\n\n");

printf("Single precision\n\n");

printf("Image name: %s\n\n",name);

printf("Save name:  %s\n\n",savename2);

printf("Psf name:   %s\n\n",kernelname);

printf("Width:  %d:",width);
prime(width);
printf("\n\n");

printf("Height: %d:",height);
prime(height);
printf("\n\n");

printf("Itmax: %6d\n\n\n",itmax);

printf("  IT         MAX       MIN        INT        FWHM      SCPU\n\n");


for (i=1; i<=2*le; i=i+2) {
   psf[i]=0.0;
   psf[i+1]=0.0;
} /* for i */

sum=0.0;

for (i=0; i<kheight; i++) {
   for (j=0; j<kwidth; j++) {

      if ((i<kworig) && (j<khorig)) {
         i2=width-(kworig-i);
         j2=height-(khorig-j);
      }
      if ((i<kworig) && (j>=khorig)) {
         i2=width-(kworig-i);
         j2=j-khorig;
      }
      if ((i>=kworig) && (j<khorig)) {
         i2=i-kworig;
         j2=height-(khorig-j);
      }
      if ((i>=kworig) && (j>=khorig)) {
         i2=i-kworig;
         j2=j-khorig;
      }
     
      fread(&dd[1],sizeof(float),1,kfile);
      ftemp=dd[1];
      
      k=2*(i2*height+j2)+1;

      psf[k]=ftemp;
      psf[k+1]=0.0;

      sum=sum+ftemp;

   } /* for j */
} /* for i */

sum=1.0/sum;

for (i=1; i<=2*le; i=i+2) {
   psf[i]=sum*psf[i];
} /* for i */



cstatistics(psf,le,stat);
printf(" PSF");
printstat(stat);
printf("\n\n");


k=1;
for (i=0; i<height; i++) {
   for (j=0; j<width; j++) {
      fread(&dd[1],sizeof(float),1,imfile);
      d0[k]=dd[1];
      k++;
   } /* for j */
} /* for i */


rstatistics(d0,le,stat);
i=0;
printf("%4d",i);
printstat(stat);
printf("\n");




tt=0;
isign=-1;
dfourt_(&psf[1],&ss[1],&ndim,&isign,&tt,&w[1]);

rrass(d1,d0,le);


isave=1;

for (it=1; it<=itmax; it++) {
 
   time1=clock();
 
   crass(dc,d1,le);
 
   tt=0;
   isign=-1;
   dfourt_(&dc[1],&ss[1],&ndim,&isign,&tt,&w[1]);

   cccmul(dc,dc,psf,le,1.0,1.0);

   tt=1;
   isign=1;
   dfourt_(&dc[1],&ss[1],&ndim,&isign,&tt,&w[1]);

   crcdiv(dc,d0,dc,le,size,eps);

   tt=0;
   isign=-1;
   dfourt_(&dc[1],&ss[1],&ndim,&isign,&tt,&w[1]);
			
   cccmul(dc,dc,psf,le,1.0,-1.0);

   tt=1;
   isign=1;
   dfourt_(&dc[1],&ss[1],&ndim,&isign,&tt,&w[1]);

   rrcmul(d1,d1,dc,le,invsize);

   rstatistics(d1,le,stat);
 
   timediff=clock()-time1;

   printf("%4d",it);
   printstat(stat);
   printf("%10.2f\n",(double)(timediff)/timetic);

   if (it == saveit[isave]) {

      strcpy(outname,savename);
      sprintf(itstr,"%d",it);
      strcat(outname,"_");
      strcat(outname,itstr);
      strcat(outname,".fff");

      outfile=fopen(outname,"wb");

      xys[1]=ss[2];
      xys[2]=ss[1];

      fwrite(&xys[1],sizeof(int),2,outfile);

      fwrite(&d1[1],sizeof(float),le,outfile);

      fclose(outfile);

      isave++;

   } /* if */

} /* for it */


printf("\n");

for (i=1; i<=savenr; i++) {
      strcpy(outname,savename);
      sprintf(itstr,"%d",saveit[i]);
      strcat(outname,"_");
      strcat(outname,itstr);
      strcat(outname,".fff");
      printf("%40s\n",outname);
} /* for i */

printf("\n");

} /* End  */
