/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*                                                           */
/*   polyfitf.c      By Stefan Spaennare                     */
/*   First version:  May 1995                                */
/*   Latest updates: Dec 1997, June 2002                     */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

#define swap(a,b) {temp=(a);(a)=(b);(b)=temp;}

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


void gaussj(a,n,b,m,indxc,indxr,ipiv)
double **a;
int n;
double **b;
int m;
int *indxc;
int *indxr;
int *ipiv;
{
   int i,icol,irow,j,k,l,ll;
   double big,dum,pivinv,temp;

   for (j=1; j<=n; j++) {
       ipiv[j]=0;
   } /* for j */

   for (i=1; i<=n; i++) {
      big=0.0;
      for (j=1; j<=n; j++) {
         if (ipiv[j] != 1) {
            for (k=1; k<=n; k++) {
               if (ipiv[k] == 0) {
                  if (fabs(a[j][k]) >= big) {
                     big=fabs(a[j][k]);
                     irow=j;
                     icol=k;
                  } /* if */
               }
               else if (ipiv[k] > 1) {
                    printf("Singular matrix 1\n");
                    exit(0);
               } /* else */
            } /* for k */
         } /* if */
      } /* for j */
      ipiv[icol]=ipiv[icol]+1;
      if (irow != icol) {
         for (l=1; l<=n; l++) {
             swap(a[irow][l],a[icol][l]);
         } /* for l */
         for (l=1; l<=m; l++) {
             swap(b[irow][l],b[icol][l]);
         } /* for l */
      } /* if */
      indxr[i]=irow;
      indxc[i]=icol;
      if (a[icol][icol] == 0.0) {
         printf("Singular matrix 2\n");
         exit(0);
      } /* if */
      pivinv=1.0/a[icol][icol];
      a[icol][icol]=1.0;
      for (l=1; l<=n; l++) {
          a[icol][l]=pivinv*a[icol][l];
      } /* for l */
      for (l=1; l<=m; l++) {
          b[icol][l]=pivinv*b[icol][l];
      } /* for l */
      for (ll=1; ll<=n; ll++) {
          if (ll != icol) {
             dum=a[ll][icol];
             a[ll][icol]=0.0;
             for (l=1; l<=n; l++) {
                 a[ll][l]=a[ll][l]-a[icol][l]*dum;
             } /* for l */
             for (l=1; l<=m; l++) {
                 b[ll][l]=b[ll][l]-b[icol][l]*dum;
             } /* for l */
          } /* if */
      } /* for ll */
   } /* for i */
  
   for (l=n; l>=1; l--) {
       if (indxr[l] != indxc[l]) {
          for (k=1; k<=n; k++) {
              swap(a[k][indxr[l]],a[k][indxc[l]]);
          } /* for k */
       } /* if */
   } /* for l */

} /* gaussj */
   

void qqsort(b,c,d,m,n)
int *b;
int *c;
double *d;
int m;
int n;
{
  int l,r,s,i,j;

  int temps;

  double d2,temp;

  int sr[65];
  int sl[65];

  s=1;
  sl[s]=m;
  sr[s]=n;

  do {
     l=sl[s];
     r=sr[s];
     s--;
    do { 
     i=l;
     j=r;
     d2=d[(l+r)/2];
     do {
        while (d[i] < d2) {
           i++;
        }
        while (d[j] > d2) {
           j--;
        }
        if (i <= j) {
           temps=b[i];
           b[i]=b[j];
           b[j]=temps;
           temps=c[i];
           c[i]=c[j];
           c[j]=temps;

           temp=d[i];
           d[i]=d[j];
           d[j]=temp;
           i++;
           j--;
        }
     } while (i <= j);
     if (i < r) {
        s++;
        sl[s]=i;
        sr[s]=r;
     }
     r=j;
    } while (l < r);
  } while (s > 0);
   
} /* qqsort */


void mshift(a,b,m,n,dx,dy) 
double **a;
double **b;
int m,n;
double dx,dy;
{
   int i,j;
   double f12,f34,f234,f1,f2,f3,f4,p21,pm2,e6;


   e6=1.0/6.0;

   p21=dx*dx-1.0;
   pm2=dx-2.0;

   f1=e6*(-dx)*(dx-1.0)*pm2;
   f2=0.5*p21*pm2;
   f3=0.5*(-dx)*(dx+1.0)*pm2;
   f4=e6*dx*p21;

   f12=f1+f2;
   f34=f3+f4;
   f234=f2+f34;

   for (j=1; j<=n; j++) {
      b[1][j]=f12*a[1][j]+f3*a[2][j]+f4*a[3][j];
      for (i=2; i<=(m-2); i++) {
         b[i][j]=f1*a[i-1][j]+f2*a[i][j]+f3*a[i+1][j]+f4*a[i+2][j];
      } /* for i */
      b[m-1][j]=f1*a[m-2][j]+f2*a[m-1][j]+f34*a[m][j];
      b[m][j]=f1*a[m-1][j]+f234*a[m][j];
   } /* for j */

   for (i=1; i<=m; i++) {
      for (j=1; j<=n; j++) {
         a[i][j]=b[i][j];
      } /* for j */
   } /* for i */

   p21=dy*dy-1.0;
   pm2=dy-2.0;

   f1=e6*(-dy)*(dy-1.0)*pm2;
   f2=0.5*p21*pm2;
   f3=0.5*(-dy)*(dy+1.0)*pm2;
   f4=e6*dy*p21;

   f12=f1+f2;
   f34=f3+f4;
   f234=f2+f34;
  
   for (i=1; i<=m; i++) {
      b[i][1]=f12*a[i][1]+f3*a[i][2]+f4*a[i][3];
      for (j=2; j<=(n-2); j++) {
         b[i][j]=f1*a[i][j-1]+f2*a[i][j]+f3*a[i][j+1]+f4*a[i][j+2];
      } /* for i */
      b[i][n-1]=f1*a[i][n-2]+f2*a[i][n-1]+f34*a[i][n];
      b[i][n]=f1*a[i][n-1]+f234*a[i][n];
   } /* for j */

} /* mshift */


void setmask(mask,xsize,ysize,xs,ys,ns,xa,ya,na,xc,yc,r0)
char **mask;
int xsize,ysize;
double *xs,*ys;
int ns;
int *xa,*ya;
int *na,*xc,*yc;
double r0;
{
   int i,j,k,xcc,ycc,xi,yi,xmaxi,ymaxi,xmini,ymini;
   double xmax,ymax,xmin,ymin,r,pi3,x0,y0,v,vs,eps;

   eps=0.0001;

   xmax=-1.0e30;
   ymax=-1.0e30;
   xmin=1.0e30;
   ymin=1.0e30;

   for (i=1; i<=ns; i++) {  
      if (xs[i] > xmax) xmax=xs[i];
      if (ys[i] > ymax) ymax=ys[i]; 
      if (xs[i] < xmin) xmin=xs[i];
      if (ys[i] < ymin) ymin=ys[i];
   } /* for i */ 

   xcc=(int)(0.5*(xmax+xmin));
   ycc=(int)(0.5*(ymax+ymin));

   xmaxi=(int)(xmax+r0+2.5);
   xmini=(int)(xmin-r0-2.5);

   ymaxi=(int)(ymax+r0+2.5);
   ymini=(int)(ymin-r0-2.5);


   if (xmaxi >= xsize) {
      xmaxi=xsize-1;
   } /* if */

   if (xmini < 0) {
      xmini=0;
   } /* if */

   if (ymaxi >= ysize) {
      ymaxi=ysize-1;
   } /* if */

   if (ymini < 0) {
      ymini=0;
   } /* if */

   for (i=xmini; i<=xmaxi; i++) {
      for (j=ymini; j<=ymaxi; j++) {
         mask[j][i]=0;
      } /* for j */
   } /* for i */

   pi3=2.0*pi+0.01*pi;

   for (i=1; i<=ns; i++) {
      x0=xs[i];
      y0=ys[i];
      r=0.0; 
      while ((r+0.5-eps) <= r0) {
         if (r < 0.5) {
	    vs=pi;
	 }
	 else {
            vs=pi/(20.0*r);
	 } /* if */
         v=0.0;
         while (v <= pi3) {
            xi=(int)(r*cos(v)+x0+0.5);
            yi=(int)(r*sin(v)+y0+0.5);
            if ((xi >= 0) && (xi < xsize)) {
               if ((yi >= 0) && (yi < ysize)) {
                  mask[yi][xi]=1;
               } /* if */
            } /* if */
            v=v+vs;
         } /* while */
         r=r+0.5;
       } /* while */
    } /* for i */

   k=1;
   for (i=xmini; i<=xmaxi; i++) {
      for (j=ymini; j<=ymaxi; j++) {
         if (mask[j][i] == 1) {
            xa[k]=i-xcc;
            ya[k]=j-ycc;
            k++;
         } /* if */
      } /* for j */
   } /* for i */

   *na=k-1;
   *xc=xcc;
   *yc=ycc;

} /* setmask */ 

void interpsf(psf0,psf1,psf2,kw,kh,x,y)
double **psf0;
double **psf1;
double **psf2;
int kw,kh;
double x,y;
{
   int i,j,xi,yi;
   double dx,dy;

   xi=(int)(x);
   yi=(int)(y);
   dx=x-(double)(xi);
   dy=y-(double)(yi);
   dx=1.0-dx;
   dy=1.0-dy; 

   for (i=1; i<=kw; i++) {
      for (j=1; j<=kh; j++) {
         psf2[i][j]=psf1[i][j];
      } /* for j */
   } /* for i */

   mshift(psf2,psf0,kw,kh,dx,dy);
  
} /* interpsf */


void initpsf(psf0,psf1,psf2,kw,kh,xs,ys,ns)
double ***psf0;
double **psf1;
double **psf2;
int kw,kh;
double *xs,*ys;
int ns;   
{
   int k;
   double x,y;

   for (k=1; k<=ns; k++) {
      x=xs[k];
      y=ys[k];

      interpsf(psf0[k],psf1,psf2,kw,kh,x,y);

   } /* for k */

} /* initpsf */


void makefit(image,psf0,kw,kh,xs,ys,ns,xa,ya,na,xc,yc,
             deg,ddeg,aa,aat,aaa,bbb,bb2,cc,t1,t2,t3)
float **image;
double ***psf0;
int kw,kh;
double *xs,*ys;
int ns;
int *xa,*ya;
int na;
int xc,yc;
int deg;
int *ddeg;
double **aa;
double **aat;
double **aaa;
double **bbb;
double *bb2;
double *cc;
int *t1,*t2,*t3;
{
   int i,j,k,n,kw2,kh2,xi,yi,n0,xi2,yi2,xp,yp,m0;
   double x,y,temp,sum;

   n0=ns+ddeg[deg];

   m0=1;
   kw2=kw/2+1;
   kh2=kh/2+1; 

   for (k=1; k<=na; k++) {

      n=n0;
      xi=xa[k];
      yi=ya[k];
      x=(double)(xi);
      y=(double)(yi);

      for (i=1; i<=ns; i++) {
         xi2=(int)(xs[i]);
         yi2=(int)(ys[i]);
 
         xp=(xi+xc)-xi2+kw2-1;
         yp=(yi+yc)-yi2+kh2-1;


         temp=0.0;

         if ((xp >= 1) && (xp <= kw)) { 
            if ((yp >= 1) && (yp <= kh)) {
               temp=psf0[i][xp][yp];
            } /* if */
         } /* if */


         aa[k][n]=temp; aat[n][k]=temp;

         n--;
      } /* for i */

      if (deg >= 5) {
         temp=x*x*x*x*x; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=y*y*y*y*y; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=x*x*x*x*y; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=x*y*y*y*y; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=x*x*x*y*y; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=x*x*y*y*y; aa[k][n]=temp; aat[n][k]=temp; n--;
      } /* if */

      if (deg >= 4) {
         temp=x*x*x*x; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=y*y*y*y; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=x*x*x*y; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=x*y*y*y; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=x*x*y*y; aa[k][n]=temp; aat[n][k]=temp; n--;
      } /* if */

      if (deg >= 3) {
         temp=x*x*x; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=y*y*y; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=x*x*y; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=x*y*y; aa[k][n]=temp; aat[n][k]=temp; n--;
      } /* if */

      if (deg >= 2) {
         temp=x*x; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=y*y; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=x*y; aa[k][n]=temp; aat[n][k]=temp; n--;
      } /* if */

      if (deg >= 1) {
         temp=x; aa[k][n]=temp; aat[n][k]=temp; n--;
         temp=y; aa[k][n]=temp; aat[n][k]=temp; n--;
      } /* if */

      if (deg >= 0) {
         temp=1.0; aa[k][n]=temp; aat[n][k]=temp; n--;
      } /* if */



      xi2=xi+xc;
      yi2=yi+yc;

      bb2[k]=(double)(image[yi2][xi2]);


   } /* for k */


   for (i=1; i<=n0; i++) {
      for (j=1; j<=n0; j++) {
         sum=0.0;
         for (k=1; k<=na; k++) {
            sum=sum+aat[i][k]*aa[k][j];
         } /* for k */
         aaa[i][j]=sum;
      } /* for j */
   } /* for i */ 


   for (j=1; j<=n0; j++) {
      sum=0.0;
      for (k=1; k<=na; k++) {
         sum=sum+aat[j][k]*bb2[k];
      } /* for k */
      bbb[j][1]=sum;
   } /* for j */


   gaussj(aaa,n0,bbb,m0,t1,t2,t3);

   for (i=1; i<=n0; i++) {
      cc[i]=bbb[i][1];
   } /* for i */

   n=n0;
   for (i=1; i<=ns; i++) {
      if (cc[n] < 0.0) {
         cc[n]=0.0;
      } /* if */
      n--; 
   } /* for i */ 

} /* makefit */ 


double ccfunc(x,y,cc,deg)
double x,y;
double *cc;
int deg;
{
   double temp;

   temp=0.0;

   if (deg >= 5) {
      temp=temp+cc[21]*x*x*x*x*x+cc[20]*y*y*y*y*y+cc[19]*x*x*x*x*y+
           cc[18]*x*y*y*y*y+cc[17]*x*x*x*y*y+cc[16]*x*x*y*y*y;
   } /* if */

   if (deg >= 4) {
      temp=temp+cc[15]*x*x*x*x+cc[14]*y*y*y*y+cc[13]*x*x*x*y+cc[12]*x*y*y*y+
           cc[11]*x*x*y*y;
   } /* if */

   if (deg >= 3) {
      temp=temp+cc[10]*x*x*x+cc[9]*y*y*y+cc[8]*x*x*y+cc[7]*x*y*y;
   } /* if */

   if (deg >= 2) {
      temp=temp+cc[6]*x*x+cc[5]*y*y+cc[4]*x*y;
   } /* if */

   if (deg >= 1) {
      temp=temp+cc[3]*x+cc[2]*y;
   } /* if */

   if (deg >= 0) {
      temp=temp+cc[1];
   } /* if */

   return(temp);

} /* ccfunc */


double getssum(image,psf0,kw,kh,xs,ys,ns,
               xa,ya,na,deg,ddeg,xc,yc,cc,flag2,sfac,bb2)
float **image;
double ***psf0;
int kw,kh;
double *xs,*ys;
int ns;
int *xa,*ya;
int *na;
int deg;
int *ddeg;
int xc,yc;
double *cc;
int flag2;
double sfac;
double *bb2;
{
   int i,k,n,kw2,kh2,xi,yi,n0,xi2,yi2,xp,yp,m0,na2;
   double x,y,temp,ssum,mean,sss,sigma,temp2;


   na2=*na;
   n0=ns+ddeg[deg];
   m0=1;
   kw2=kw/2+1;
   kh2=kh/2+1; 
   ssum=0.0;
   mean=0.0;

   for (k=1; k<=na2; k++) {

      n=n0;
      xi=xa[k];
      yi=ya[k];
      x=(double)(xi);
      y=(double)(yi);
      temp=0.0;

      for (i=1; i<=ns; i++) {
         xi2=(int)(xs[i]);
         yi2=(int)(ys[i]);
 
         xp=(xi+xc)-xi2+kw2-1;
         yp=(yi+yc)-yi2+kh2-1;


         if ((xp >= 1) && (xp <= kw)) { 
            if ((yp >= 1) && (yp <= kh)) {
               temp=temp+cc[n]*psf0[i][xp][yp];
            } /* if */
         } /* if */

         n--;
      } /* for i */

      temp=temp+ccfunc(x,y,cc,deg);

      xi2=xi+xc;
      yi2=yi+yc;

      temp=temp-(double)(image[yi2][xi2]);

      mean=mean+temp;
      ssum=ssum+temp*temp;

   } /* for k */


if (flag2 == 1) {

   mean=mean/(double)(na2);
   sss=0.0;


   for (k=1; k<=na2; k++) {

      n=n0;
      xi=xa[k];
      yi=ya[k];
      x=(double)(xi);
      y=(double)(yi);
      temp=0.0;

      for (i=1; i<=ns; i++) {
         xi2=(int)(xs[i]);
         yi2=(int)(ys[i]);
 
         xp=(xi+xc)-xi2+kw2-1;
         yp=(yi+yc)-yi2+kh2-1;


         if ((xp >= 1) && (xp <= kw)) { 
            if ((yp >= 1) && (yp <= kh)) {
               temp=temp+cc[n]*psf0[i][xp][yp];
            } /* if */
         } /* if */

         n--;
      } /* for i */

      temp=temp+ccfunc(x,y,cc,deg);

      xi2=xi+xc;
      yi2=yi+yc;

      temp=temp-(double)(image[yi2][xi2]);

      temp2=temp-mean;

      bb2[k]=fabs(temp2);

      sss=sss+temp2*temp2;

   } /* for k */


   sigma=sfac*sqrt(sss/((double)(na2)-1.0));

   qqsort(xa,ya,bb2,1,na2);

   i=1;

   while ((bb2[i] < sigma) && (i < na2)) {
      i++;
   } /* while */

   *na=i;

   na2=i;

   ssum=0.0;

   for (k=1; k<=na2; k++) {

      n=n0;
      xi=xa[k];
      yi=ya[k];
      x=(double)(xi);
      y=(double)(yi);
      temp=0.0;

      for (i=1; i<=ns; i++) {
         xi2=(int)(xs[i]);
         yi2=(int)(ys[i]);
 
         xp=(xi+xc)-xi2+kw2-1;
         yp=(yi+yc)-yi2+kh2-1;


         if ((xp >= 1) && (xp <= kw)) { 
            if ((yp >= 1) && (yp <= kh)) {
               temp=temp+cc[n]*psf0[i][xp][yp];
            } /* if */
         } /* if */

         n--;
      } /* for i */

      temp=temp+ccfunc(x,y,cc,deg);

      xi2=xi+xc;
      yi2=yi+yc;

      temp=temp-(double)(image[yi2][xi2]);

      ssum=ssum+temp*temp;

   } /* for k */

} /* if */
  

   ssum=sqrt(ssum/(double)(na2));

   return(ssum);

} /* getssum */ 



void getint(psf0,kw,kh,inte,ns,deg,ddeg,cc)
double ***psf0;
int kw,kh;
double *inte;
int ns;
int deg;
int *ddeg;
double *cc;
{
   int i,j,k,n;
   double sum;

   n=ns+ddeg[deg];

   for (k=1; k<=ns; k++) {

      sum=0.0;

      for (i=1; i<=kw; i++) {
         for (j=1; j<=kh; j++) {
            sum=sum+psf0[k][i][j];
         } /* for j */
      } /* for i */
   
      inte[k]=cc[n]*sum;

      n--;

   } /* for k */

} /* getint */
     

int chkcoord(xs,ys,ns,x,y,s,ceps)
double *xs,*ys;
int ns;
double x,y;
int s;
double ceps; 
{
   int i,flag;

   double dx,dy,eps2,d;

   eps2=ceps*ceps;
   
   flag=0;
   i=1;
  
   while ((i <= ns) && (flag == 0)) { 
      if (i != s) {
         dx=xs[i]-x;
         dy=ys[i]-y;
         d=dx*dx+dy*dy;
         if (d < eps2)  {
            flag=1;
         } /* if */
      } /* if */
      i++;
   } /* while */

   return(flag);

} /* chkcoord */


int main(argc,argv) 
int argc;
char *argv[];

{
   
   FILE *infile,*imfile,*psffile,*outfile;

   double x,y,sum,ftemp;
   double xystep0,r0,ssum,xyeps,inte1,xystep,xystep1;
   double xrem,yrem,x0,y0,ssum0,ssummin,xyfac,ceps,ftime,mag,lt,sss;
   double sfac,amp1;

   int i,j,k,l;
   int xsize,ysize; 

   int kh,kw,kw1,kh1;
   int maxns,deg,ns;
   int naa,maa,xc,yc,na,flag,it,grids,gs,xyit,oaa,nss;
   int n,kh0,kw0,khs,kws;

   int ddeg[7];

   clock_t time1,timediff;

   double **psf1,**psf2;
   double *mpsf1,*mpsf2;

   double ***psf0;
   double **mppsf0;
   double  *mmppsf0;

   double **aa,**aat;
   double *mpaa, *mpaat;

   double **aaa;
   double  *mpaaa;

   double **bbb;
   double  *mpbbb;

   char **mask;
   char *mpmask;

   double *cc;

   double *bb2;

   double *xs,*ys;

   double *inte;

   int *xa,*ya;

   int *t1,*t2,*t3;

   int xys[3];

   float **image;
   float *mpimage;

   float **psf;
   float *mppsf;

   float fff[3];



if (argc != 15) {
   printf("Usage: %s image.fff psf.fff maxnstarsi polydegi xystepr xyfacr xyepsr\n",argv[0]);
   printf("                gridsi rfitr sigmafacr coordepsr ampr groupfile outfile \n");
   exit(0);
} /* if */


maxns=atoi(argv[3]);
deg=atoi(argv[4]);
xystep0=atof(argv[5]);
xyfac=atof(argv[6]);
xyeps=atof(argv[7]);
grids=atoi(argv[8]);
r0=atof(argv[9]);
sfac=atof(argv[10]);
ceps=atof(argv[11]);
amp1=atof(argv[12]);

xyit=1;


if ((infile = fopen(argv[13],"r")) == NULL) {
   printf("File %s not found.\n",argv[13]);
   exit(0);
} /* if */


if ((imfile = fopen(argv[1],"rb")) == NULL) {
   printf("File %s not found.\n",argv[1]);
   exit(0);
} /* if */


outfile=fopen(argv[14],"w");


fread(&xys[1],sizeof(int),2,imfile);

xsize=xys[1];
ysize=xys[2];

image=(float **)calloc(ysize,sizeof(float*));
mpimage=(float *)calloc(xsize*ysize,sizeof(float));

for (i=0; i<ysize; i++) {
   image[i]=mpimage;
   mpimage=mpimage+xsize;
} /* for i */

for (i=0; i<ysize; i++) {
   for (j=0; j<xsize; j++) {
      fread(&fff[1],sizeof(float),1,imfile);
      image[i][j]=fff[1];
   } /* for j */
} /* for i */   

fclose(imfile);

if ((psffile = fopen(argv[2],"rb")) == NULL) {
   printf("File %s not found.\n",argv[2]);
   exit(0);
} /* if */

fread(&xys[1],sizeof(int),2,psffile);

kw=xys[1];
kh=xys[2];

psf=(float **)calloc(kh,sizeof(float*));
mppsf=(float *)calloc(kw*kh,sizeof(float));

for (i=0; i<kh; i++) {
   psf[i]=mppsf;
   mppsf=mppsf+kw;
} /* for i */

for (i=0; i<kh; i++) {
   for (j=0; j<kw; j++) {
      fread(&fff[1],sizeof(float),1,psffile);
      psf[i][j]=fff[1];
   } /* for j */
} /* for i */   

fclose(psffile);


ddeg[0]=1; ddeg[1]=3; ddeg[2]=6; ddeg[3]=10; ddeg[4]=15; ddeg[5]=21;

kw0=kw;
kh0=kh;
kw1=kw+1;
kh1=kh+1;

mask=(char **)calloc(ysize,sizeof(char*));
mpmask=(char *)calloc(xsize*ysize,sizeof(char));

for (i=0; i<ysize; i++) {
   mask[i]=mpmask;
   mpmask=mpmask+xsize;
} /* for i */

psf1=(double **)calloc(kw1+1,sizeof(double*));
mpsf1=(double *)calloc((kw1+1)*(kh1+1),sizeof(double));

psf2=(double **)calloc(kw1+1,sizeof(double*));
mpsf2=(double *)calloc((kw1+1)*(kh1+1),sizeof(double));

for (i=0; i<=kw1; i++) {
   psf1[i]=mpsf1;
   psf2[i]=mpsf2;
   mpsf1=mpsf1+kh1+1;
   mpsf2=mpsf2+kh1+1;
} /* for i */
 
psf0=(double ***)calloc(maxns+1,sizeof(double**));
mppsf0=(double **)calloc((maxns+1)*(kw1+1),sizeof(double*));
mmppsf0=(double *)calloc((maxns+1)*(kw1+1)*(kh1+1),sizeof(double));

for (i=0; i<=maxns; i++) {
   psf0[i]=mppsf0;
   for (j=0; j<=kw1; j++) {
      mppsf0[j]=mmppsf0;
      mmppsf0=mmppsf0+kh1+1;
   } /* for j*/
   mppsf0=mppsf0+kw1+1;
} /* for i */


naa=maxns+ddeg[deg];

maa=maxns*(int)(pi*(r0+1.0)*(r0+1.0)+0.5);

oaa=maxns*(int)(pi*(r0+1.0)*(r0+1.0)+0.5);

xa=(int *)calloc(oaa+1,sizeof(int));
ya=(int *)calloc(oaa+1,sizeof(int));

t1=(int *)calloc(naa+1,sizeof(int));
t2=(int *)calloc(naa+1,sizeof(int));
t3=(int *)calloc(naa+1,sizeof(int));

aaa=(double **)calloc(naa+1,sizeof(double*));
mpaaa=(double *)calloc((naa+1)*(naa+1),sizeof(double));

for (i=0; i<=naa; i++) {
   aaa[i]=mpaaa;
   mpaaa=mpaaa+naa+1;
} /* for i */

bbb=(double **)calloc(naa+1,sizeof(double*));
mpbbb=(double *)calloc((naa+1)*2,sizeof(double));

for (i=0; i<=naa; i++) {
   bbb[i]=mpbbb;
   mpbbb=mpbbb+2;
} /* for i */

aa=(double **)calloc(maa+1,sizeof(double*));
aat=(double **)calloc(naa+1,sizeof(double*));

mpaa=(double *)calloc((maa+1)*(naa+1),sizeof(double));
mpaat=(double *)calloc((maa+1)*(naa+1),sizeof(double));

for (i=0; i<=maa; i++) {
   aa[i]=mpaa;
   mpaa=mpaa+naa+1;
} /* for i */

for (i=0; i<=naa; i++) {
   aat[i]=mpaat; 
   mpaat=mpaat+maa+1;
} /* for i */

cc=(double *)calloc(naa+1,sizeof(double));

bb2=(double *)calloc(maa+1,sizeof(double));

xs=(double *)calloc(maxns+1,sizeof(double));
ys=(double *)calloc(maxns+1,sizeof(double));
inte=(double *)calloc(maxns+1,sizeof(double));



sum=0.0;

for (i=0; i<kw0; i++) {
   for (j=0; j<kh0; j++) {
      ftemp=(double)(psf[j][i]);
      sum=sum+ftemp;
   } /* for j */
} /* for i */

sum=1.0/sum;

khs=(kh0-kh)/2;
kws=(kw0-kw)/2;

for (i=0; i<kw; i++) {
   for (j=0; j<kh; j++) {
      ftemp=(double)(psf[j+khs][i+kws]);
      psf1[i+1][j+1]=sum*ftemp;
   } /* for j */
} /* for i */



flag=0;
nss=0;

do {

i=1;
do {
   fscanf(infile,"%lf %lf %lf",&x,&y,&inte1);
   if (x > -0.5) {
      xs[i]=x;
      ys[i]=y;
      i++;
   } /* if */
} while (x > -0.5); 

ns=i-1;

if (x < -2.0) {
   flag=1;
} /* if */

if (flag == 0) {

time1=clock();

setmask(mask,xsize,ysize,xs,ys,ns,xa,ya,&na,&xc,&yc,r0);

initpsf(psf0,psf1,psf2,kw,kh,xs,ys,ns);

makefit(image,psf0,kw,kh,xs,ys,ns,xa,ya,na,xc,yc,deg,ddeg,
        aa,aat,aaa,bbb,bb2,cc,t1,t2,t3);

ssum0=getssum(image,psf0,kw,kh,xs,ys,ns,
              xa,ya,&na,deg,ddeg,xc,yc,cc,1,sfac,bb2);

ssummin=ssum0;

xystep=xystep0;

gs=grids/2;

it=1;

while (xystep > xyeps) {


   xystep1=xystep/(double)(gs);

   for (i=1; i<=ns; i++) {

       x0=xs[i];
       y0=ys[i];

       xrem=x0;
       yrem=y0;

       for (k=-gs; k<=gs; k++) {
          for (l=-gs; l<=gs; l++) {
             x=(double)(k)*xystep1+x0;
             y=(double)(l)*xystep1+y0;

             if (chkcoord(xs,ys,ns,x,y,i,ceps) == 0) {

                xs[i]=x;
                ys[i]=y;

                interpsf(psf0[i],psf1,psf2,kw,kh,x,y);

                makefit(image,psf0,kw,kh,xs,ys,ns,
                        xa,ya,na,xc,yc,deg,ddeg,
                        aa,aat,aaa,bbb,bb2,cc,t1,t2,t3);

                ssum=getssum(image,psf0,kw,kh,xs,ys,ns,
                           xa,ya,&na,deg,ddeg,xc,yc,cc,0,sfac,bb2);
             

                if (ssum < ssummin) {
                   ssummin=ssum;
                   xrem=x;
                   yrem=y;
                } /* if */

             } /* if */

          } /* for l */
       } /* for k */


       xs[i]=xrem;
       ys[i]=yrem;


       x=xs[i];
       y=ys[i];

       interpsf(psf0[i],psf1,psf2,kw,kh,x,y);


    } /* for i */


    setmask(mask,xsize,ysize,xs,ys,ns,xa,ya,&na,&xc,&yc,r0);


    makefit(image,psf0,kw,kh,xs,ys,ns,
            xa,ya,na,xc,yc,deg,ddeg,
            aa,aat,aaa,bbb,bb2,cc,t1,t2,t3);


    ssummin=getssum(image,psf0,kw,kh,xs,ys,ns,
                    xa,ya,&na,deg,ddeg,xc,yc,cc,1,sfac,bb2);


    if (it == xyit) {
       xystep=xyfac*xystep;
       it=0;
    } /* if */

    it++;
} /* while */


sss=ssummin;

timediff=clock()-time1;
ftime=(double)(timediff)/timetic;

lt=1.0/log(10.0);

n=ns+ddeg[deg];

for (i=1; i<=ns; i++) {

   inte[i]=amp1*cc[n+1-i];
   mag=0.0;

   if (inte[i] > 1.0) {
      mag=-2.5*lt*log(inte[i]);
   } /* if */

   fprintf(outfile,"%5d %8.3f %8.3f %13.2f %8.3f %8.2f %11.5f %6.2f\n",
          nss+i,xs[i],ys[i],inte[i],mag,amp1*cc[1],amp1*sss,ftime);

} /* for i */

nss=nss+ns;

fprintf(outfile,"\n");

} /* if */

} while (flag == 0);

printf("   nr     xpos     ypos           int      mag       bg       sigma   scpu\n");
printf("\n");
printf("   Printed to file: %s\n",argv[14]);

fclose(infile);
fclose(outfile);

} /* End  */
