/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  setmoff.c  SS Sept 93                                    */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h> 
#include <stdlib.h>
#include <unistd.h>
#include "image.h"

int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header; 
   short **image;


   double x,y,z,pi,ny,fwhm;
   double a,c,gamma,forigx,forigy;

   int i,j,width,height,re,ksize;



   if (argc<5) {
      printf("Usage: %s image sizei fwhmr betar\n",argv[0]);
      exit(0);
   } /* if */

   
   ksize=atoi(argv[2]);
   fwhm=atof(argv[3]);
   ny=atof(argv[4]);


   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=ksize;
   header.wid=ksize;


   image = (short **)calloc(header.hgh,sizeof(long));

   CreateIm_s16(image,header.hgh,header.wid);


   pi=4.0*atan(1.0);

   width=header.wid;
   height=header.hgh;
   forigx=(double)(width-1)/2.0;
   forigy=(double)(height-1)/2.0;

   gamma=4.0*(exp(log(2.0)/ny)-1.0);
   a=gamma/(fwhm*fwhm);
   c=gamma/(fwhm*fwhm);

   for (i=0; i<width; i++) {
      for (j=0; j<height; j++) {

         x=(double)(i)-forigx;
         y=(double)(j)-forigy;

         re=(int)(32767.0*exp(-ny*log(1.0+a*x*x+c*y*y))+0.5);
         
         image[j][i]=re; 

      } /* for j */
   } /* for i */  


   WriteImage(argv[1],image,header,im_s16);

} /* End setpsf */
