/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  imf2fit.c   SS Oct 95, Dec 97                           */
/*  LINUX version                                            */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "image.h"


int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header; 
   short **image;

   FILE *outfile;
   
   int i,j,k,n,rgb,xsize,ysize,temp,n2,rn,n3;
   int n4,h,l,n33;
  
   short *d;

   unsigned char *ss;


   if (argc<3) {
      printf("Usage: %s image image.fit\n",argv[0]);
      exit(0);
   } /* if */

 
   ReadHeader(argv[1],&header);

   image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],image);

   xsize=header.wid;
   ysize=header.hgh;

   outfile = fopen(argv[2],"wb");

   ss=(char *)calloc(81,sizeof(char));


   sprintf(ss,"%s","SIMPLE  =                    T");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s","BITPIX  =                   16");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s","NAXIS   =                    2");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s%6d","NAXIS1  =               ",xsize);

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s%6d","NAXIS2  =               ",ysize);

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s","BSCALE  =       1.0000000000E0");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s","BZERO   =       0.0000000000E0");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);



   sprintf(ss,"%s","END                           ");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);


   for (i=0; i<=81; i++) {
      ss[i]=' ';
   } 

   for (i=1; i<=28; i++) {
      fwrite(ss,sizeof(char),80,outfile);
   }


   printf("%6d %6d\n",xsize,ysize);

   n=xsize*ysize;

   n2=2*n;

   n3=n2 / 2880;

   n3=(n3+1)*2880;
 
   n33=n3/2;

   n4=(n3-n2)/2;

   d = (short *)calloc(n33+1,sizeof(short));

   k=1;

   for (i=ysize-1; i>=0; i--) {
      for (j=0; j<xsize; j++) {
          temp=image[i][j];
          h=temp / 256;
          l=temp % 256;
          d[k]=256*l+h;
          k++;
      } /* for j */
   } /* for i */

   for (i=1; i<=n4; i++) {
       d[k]=0;
       k++;
   } /* for i */

   fwrite(&d[1],sizeof(short),n33,outfile);

   fclose(outfile);

} /* End  */
