/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/******************************************************************/
/*                                                                */
/* Program: imf2tgal.c                                            */
/*                                                                */
/* Description:                                                   */
/*                                                                */
/* This program converts a 16 bit GOP greyscale-image (*.imf) to  */
/* to a 24 bit RGB Targa file (*.tga).                            */
/* Linux version.                                                 */
/*                                                                */
/* Program by: Stefan Spaennare, Lund, October 1995               */
/* Email:      stefans@astro.lu.se                                */
/* Latest update: 1997-11-27                                      */
/*                                                                */
/******************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "image.h"


int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header; 
   short **image;

   FILE *outfile;
   
   int i,j,k,n,rgb,xsize,ysize,temp;
  
   unsigned char d[4];

   unsigned short ss[10];


   if (argc<3) {
      printf("Usage: %s gopimage image.tga\n",argv[0]);
      exit(0);
   } /* if */

 
   ReadHeader(argv[1],&header);

   image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],image);

   outfile = fopen(argv[2],"wb");

   ss[1]=0;
   ss[2]=2;
   ss[3]=0;
   ss[4]=0;
   ss[5]=0;
   ss[6]=0;
   ss[7]=header.wid;
   ss[8]=header.hgh;
   ss[9]=8216;


   fwrite(&ss[1],sizeof(unsigned short),9,outfile);

   printf("%6d %6d\n",ss[7],ss[8]);
 
   xsize=ss[7];
   ysize=ss[8];

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
          temp=image[i][j]/128;
          d[1]=temp;
          d[2]=temp;
          d[3]=temp;
          fwrite(&d[1],sizeof(unsigned char),3,outfile);
      } /* for j */
   } /* for i */


   fclose(outfile);


} /* End  */
