/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/******************************************************************/
/*                                                                */
/* Program: imf2tgau2.c                                           */
/*                                                                */
/* Description:                                                   */
/*                                                                */
/* This program converts 3 8-bit GOP images (*.imf) to a 24 bit   */
/* RGB Targa file (*.tga). The GOP-images should be labeled       */
/* *R, *G and *B, where * is an arbitrary name.                   */
/* UNIX version.                                                  */
/*                                                                */
/* Program by: Stefan Spaennare, Lund, October 1995               */
/* Email:      stefans@astro.lu.se                                */
/* Latest update: 1997-11-27                                      */
/*                                                                */
/******************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "image.h"


int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header1,header2,header3; 
   unsigned char **image1,**image2,**image3;

   FILE *outfile;
   
   int i,j,k,n,rgb,xsize,ysize,temp;
   int hhh,lll;
  
   unsigned char d[4];

   unsigned short ss[10];

   char nameR[80];
   char nameG[80];
   char nameB[80];


   if (argc<3) {
      printf("Usage: %s gopimage(RGB) image.tga\n",argv[0]);
      exit(0);
   } /* if */

   strcpy(nameR,argv[1]);
   strcat(nameR,"R");
   strcpy(nameG,argv[1]);
   strcat(nameG,"G");
   strcpy(nameB,argv[1]);
   strcat(nameB,"B");

 
   ReadHeader(nameR,&header1);

   image1 = (unsigned char **)calloc(header1.hgh,sizeof(unsigned char*));

   ReadImage(nameR,image1);

   ReadHeader(nameG,&header2);

   image2 = (unsigned char **)calloc(header2.hgh,sizeof(unsigned char*));

   ReadImage(nameG,image2);

   ReadHeader(nameB,&header3);

   image3 = (unsigned char **)calloc(header3.hgh,sizeof(unsigned char*));

   ReadImage(nameB,image3);


   outfile = fopen(argv[2],"wb");

   xsize=header1.wid;
   ysize=header1.hgh;


   ss[1]=0;
   ss[2]=512;
   ss[3]=0;
   ss[4]=0;
   ss[5]=0;
   ss[6]=0;
   hhh=xsize / 256;
   lll=xsize % 256;
   ss[7]=lll*256+hhh;
   hhh=ysize / 256;
   lll=ysize % 256;
   ss[8]=lll*256+hhh;
   ss[9]=6176;

   fwrite(&ss[1],sizeof(unsigned short),9,outfile);

   printf("%6d %6d\n",xsize,ysize);
 
   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
          d[1]=image3[i][j];
          d[2]=image2[i][j];
          d[3]=image1[i][j];
          fwrite(&d[1],sizeof(unsigned char),3,outfile);
      } /* for j */
   } /* for i */

   fclose(outfile);

} /* End  */
