/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  imlog.c  SS Aug 96                                       */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "image.h"


int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header; 
   short **image;

   int i,j,z;

   double fac,lt,temp,max;

   if (argc<3) {
      printf("Usage: %s inimage outimage ampr\n",argv[0]);
      exit(0);
   } /* if */

   fac=atof(argv[3]);

 
   ReadHeader(argv[1],&header);

   image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],image);

   lt=1.0/log(10.0);

   max=-1.0e30;

   for (i=0; i<header.hgh; i++) {
      for (j=0; j<header.wid; j++) {
         temp=log((double)(image[i][j])+1.0);
	 if (temp > max) {
	    max=temp;
	 } /* if */
      } /* for j */
   } /* for i */

   fac=lt*fac*32767.0/max;

   for (i=0; i<header.hgh; i++) {
      for (j=0; j<header.wid; j++) {
         z=(int)(fac*log((double)(image[i][j])+1.0)+0.5);
         if (z > 32767) {
            z=32767;
         } /* if */
         image[i][j]=z;
      } /* for j */
   } /* for i */

   WriteImage(argv[2],image,header,im_s16);

} /* End  */
