/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*   imstarsf.c    SS May 95 Dec 97                          */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


void mshift(a,b,m,n,dx,dy) 
double **a;
double **b;
int m,n;
double dx,dy;
{
   int i,j;
   double f12,f34,f234,f1,f2,f3,f4,p21,pm2,e6;


   e6=1.0/6.0;

   p21=dx*dx-1.0;
   pm2=dx-2.0;

   f1=e6*(-dx)*(dx-1.0)*pm2;
   f2=0.5*p21*pm2;
   f3=0.5*(-dx)*(dx+1.0)*pm2;
   f4=e6*dx*p21;

   f12=f1+f2;
   f34=f3+f4;
   f234=f2+f34;

   for (j=1; j<=n; j++) {
      b[1][j]=f12*a[1][j]+f3*a[2][j]+f4*a[3][j];
      for (i=2; i<=(m-2); i++) {
         b[i][j]=f1*a[i-1][j]+f2*a[i][j]+f3*a[i+1][j]+f4*a[i+2][j];
      } /* for i */
      b[m-1][j]=f1*a[m-2][j]+f2*a[m-1][j]+f34*a[m][j];
      b[m][j]=f1*a[m-1][j]+f234*a[m][j];
   } /* for j */

   for (i=1; i<=m; i++) {
      for (j=1; j<=n; j++) {
         a[i][j]=b[i][j];
      } /* for j */
   } /* for i */

   p21=dy*dy-1.0;
   pm2=dy-2.0;

   f1=e6*(-dy)*(dy-1.0)*pm2;
   f2=0.5*p21*pm2;
   f3=0.5*(-dy)*(dy+1.0)*pm2;
   f4=e6*dy*p21;

   f12=f1+f2;
   f34=f3+f4;
   f234=f2+f34;
  
   for (i=1; i<=m; i++) {
      b[i][1]=f12*a[i][1]+f3*a[i][2]+f4*a[i][3];
      for (j=2; j<=(n-2); j++) {
         b[i][j]=f1*a[i][j-1]+f2*a[i][j]+f3*a[i][j+1]+f4*a[i][j+2];
      } /* for i */
      b[i][n-1]=f1*a[i][n-2]+f2*a[i][n-1]+f34*a[i][n];
      b[i][n]=f1*a[i][n-1]+f234*a[i][n];
   } /* for j */

} /* mshift */




int main(argc,argv) 
int argc;
char *argv[];

{

   FILE *infile,*imfile,*kfile;

   double amp0,max,diffsec,sum,ftemp,ampr;
   double x,y,inte,dx,dy,rmax2,maxlev;

   float fftemp;

   int i,j,k,l,k2;
   int xsize,ysize; 
   int xi,yi,i2,j2;

   int kh,kw,kw1,kh1,kw2,kh2;

   int xys[3];
   int kxys[3];


   char name[80];
   char psfname[80];
   char filename[80];


   double **psf0,**psf1,**psf2;

   double *mpsf0,*mpsf1,*mpsf2;

   float **fimage;

   float *mpfimage;

   float dd[3];
 
   clock_t time1,timediff;



if (argc < 6) {
printf("Usage: %s image.fff xsizei ysizei psf.fff ampr coord(x y int)\n",argv[0]);
   exit(0);
} /* if */


strcpy(name,argv[1]);
xsize=atoi(argv[2]);
ysize=atoi(argv[3]);
strcpy(psfname,argv[4]);
amp0=atof(argv[5]);
strcpy(filename,argv[6]);


if ((infile = fopen(filename,"r")) == NULL) {
   printf("File not found.\n");
   exit(0);
} /* if */

imfile = fopen(name,"wb");

if ((kfile = fopen(psfname,"rb")) == NULL) {
   printf("File not found.\n");
   exit(0);
} /* if */


fread(&kxys[1],sizeof(int),2,kfile);

kw=kxys[1];
kh=kxys[2];

kw1=kw+1;
kh1=kh+1;

psf0=(double **)calloc(kw1+1,sizeof(double*));
mpsf0=(double *)calloc((kw1+1)*(kh1+1),sizeof(double));

psf1=(double **)calloc(kw1+1,sizeof(double*));
mpsf1=(double *)calloc((kw1+1)*(kh1+1),sizeof(double));

psf2=(double **)calloc(kw1+1,sizeof(double*));
mpsf2=(double *)calloc((kw1+1)*(kh1+1),sizeof(double));

for (i=0; i<=kw1; i++) {
   psf0[i]=mpsf0;
   psf1[i]=mpsf1;
   psf2[i]=mpsf2;
   mpsf0=mpsf0+kh1+1;
   mpsf1=mpsf1+kh1+1;
   mpsf2=mpsf2+kh1+1;
} /* for i */
 

fimage=(float **)calloc(ysize,sizeof(float*));
mpfimage=(float *)calloc(xsize*ysize,sizeof(float));

for (i=0; i<ysize; i++) {
   fimage[i]=mpfimage;
   mpfimage=mpfimage+xsize;
} /* for i */


for (j=0; j<ysize; j++) {
   for (i=0; i<xsize; i++) {
      fimage[j][i]=0.0;
   } /* for j */
} /* for i */

for (i=1; i<=kh; i++) {
   for (j=1; j<=kw; j++) {
       fread(&dd[1],sizeof(float),1,kfile);
       psf0[j][i]=(double)(dd[1]);
   } /* for j */
} /* for i */


kw2=kw/2+1;
kh2=kh/2+1;


time1=clock();


while (fscanf(infile,"%lf %lf %lf",&x,&y,&inte) != EOF) {

      xi=(int)(x);
      yi=(int)(y);
      dx=x-(double)(xi);
      dy=y-(double)(yi);

      dx=1.0-dx;
      dy=1.0-dy;

      for (i=1; i<=kh; i++) {
         for (j=1; j<=kw; j++) {
            psf1[i][j]=psf0[i][j];
         } /* for j */
      } /* for i */


      mshift(psf1,psf2,kw,kh,dx,dy);


      sum=0.0;

      for (i=1; i<=kh; i++) {
         for (j=1; j<=kw; j++) {
             sum=sum+psf2[i][j];
         } /* for j */
      } /* for i */

      sum=amp0*inte/sum;

      for (i=1; i<=kh; i++) {
         for (j=1; j<=kw; j++) {
             i2=i+xi-kw2+1;
             j2=j+yi-kh2+1;
             if ((i2 >= 0) && (i2 < xsize)) {
                if ((j2 >= 0) && (j2 < ysize)) {
                   fftemp=fimage[j2][i2]+(float)(sum*psf2[i][j]);
                   fimage[j2][i2]=fftemp;
                } /* if */
             } /* if */
         } /* for j */
      } /* for i */
      

} /* while */
 
xys[1]=xsize;
xys[2]=ysize;


fwrite(&xys[1],sizeof(int),2,imfile);
 

for(j=0; j<ysize; j++) {
   for (i=0; i<xsize; i++) {
      dd[1]=fimage[j][i];
      fwrite(&dd[1],sizeof(float),1,imfile);
   } /* for j */
} /* for i */

timediff=clock()-time1;

printf("%10.2f\n",(double)(timediff)/timetic);

fclose(infile);
fclose(imfile);
fclose(kfile);

} /* End  */
