/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/***********************************************************************************/
/*                                                                                 */
/* Program:                                                                        */
/* ========                                                                        */
/*                                                                                 */
/* movav.c                                                                        */
/*                                                                                 */
/*                                                                                 */
/* Version:                                                                        */
/* ========                                                                        */
/*                                                                                 */
/* First version: 2003-03-07                                                       */
/* Latest update: 2006-11-13                                                       */
/*                                                                                 */
/*                                                                                 */
/* Author:                                                                         */
/* =======                                                                         */
/*                                                                                 */
/* Program by: Stefan Spaennare                                                    */
/* Email:      stefans@astro.lu.se                                                 */ 
/* Web-page:   http://www.astro.lu.se/~stefans/aprog.html                          */
/*                                                                                 */
/***********************************************************************************/     

/***********************************************************************************/
/*                                                                                 */
/* Notice:                                                                         */
/* =======                                                                         */
/*                                                                                 */
/* I make no warranties that this program is (1) free of error, (2) con-           */
/* sistent with any standard merchantability, or (3) meeting the require-          */
/* ments of a particular application.  This software shall not, partly or          */
/* as a whole, participate in a process, whose outcome can result in injury        */
/* to a person or loss of property. It is solely designed for analytical           */
/* work.  Permission to use, copy, and distribute is hereby granted without        */
/* fee, providing that the header above including this notice appears in           */
/* all copies.                                                                     */
/*                                                                                 */
/*                                                         Stefan Spaennare        */
/*                                                                                 */
/***********************************************************************************/


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>


double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


void qqsort2(d,e,m,n)
double *d;
double *e;
int m;
int n;
{
  int l,r,s,i,j;

  double d2,temp;

  int sr[65];
  int sl[65];

  s=1;
  sl[s]=m;
  sr[s]=n;

  do {
     l=sl[s];
     r=sr[s];
     s--;
    do { 
     i=l;
     j=r;
     d2=d[(l+r)/2];
     do {
        while (d[i] < d2) {
           i++;
        } /* while */
        while (d[j] > d2) {
           j--;
        } /* while */
        if (i <= j) {
           temp=d[i];
           d[i]=d[j];
           d[j]=temp;
           temp=e[i];
           e[i]=e[j];
           e[j]=temp;
           i++;
           j--;
        } /* if */
     } while (i <= j);
     if (i < r) {
        s++;
        sl[s]=i;
        sr[s]=r;
     } /* if */
     r=j;
    } while (l < r);
  } while (s > 0);
   
} /* qqsort2 */


void qqsort(d,m,n)
double *d;
int m;
int n;
{
  int l,r,s,i,j;

  double d2,temp;

  int sr[65];
  int sl[65];

  s=1;
  sl[s]=m;
  sr[s]=n;

  do {
     l=sl[s];
     r=sr[s];
     s--;
    do { 
     i=l;
     j=r;
     d2=d[(l+r)/2];
     do {
        while (d[i] < d2) {
           i++;
        } /* while */
        while (d[j] > d2) {
           j--;
        } /* while */
        if (i <= j) {
           temp=d[i];
           d[i]=d[j];
           d[j]=temp;
           i++;
           j--;
        } /* if */
     } while (i <= j);
     if (i < r) {
        s++;
        sl[s]=i;
        sr[s]=r;
     } /* if */
     r=j;
    } while (l < r);
  } while (s > 0);
   
} /* qqsort */



int main(argc,argv) 
int argc;
char *argv[];

{
   
   FILE *infile,*outfile;

   int n,nn,i,j,nint,m,mm,mstep;
   int imed1,imed2,imed3;

   double inte1,inte2;
   double lt,mag0,fi1,gauss1,gauss2;
   double median,siglow,sighigh;

   double *mag;
   double *dmag;
   double *dmag2;

   clock_t time1,timediff1;

   if (argc != 5) {
      printf("Usage: %s infile(int1 int2) outfile mi mstepi\n",argv[0]);
      exit(0);
   } /* if */
   

   if ((infile = fopen(argv[1],"r")) == NULL) {
      printf("File %s not found.\n",argv[1]);
      exit(0);
   } /* if */

   outfile=fopen(argv[2],"w");

   m=atoi(argv[3]);
   mstep=atoi(argv[4]);

   lt=1.0/log(10.0);


   i=1;

   while (fscanf(infile,"%lf %lf",&inte1,&inte2) != EOF) {
      i++;
   } /* while */

   n=i-1;

   nn=n/nint;

   mag=(double *)calloc(n+1,sizeof(double));
   dmag=(double *)calloc(n+1,sizeof(double));
   dmag2=(double *)calloc(m+1,sizeof(double));

   rewind(infile);

   i=1;

   while (fscanf(infile,"%lf %lf",&inte1,&inte2) != EOF) {
      mag[i]=-2.5*lt*log(inte1);
      dmag[i]=mag[i]-(-2.5*lt*log(inte2));
      i++;
   } /* while */

   qqsort2(mag,dmag,1,n);

   fi1=0.841344746068542948585;
   gauss1=fi1-(1.0-fi1);
   gauss2=0.5*gauss1;

   imed1=(int)((double)(m)*(0.5-gauss2)+0.5);
   imed2=(int)((double)(m)*(0.5)+0.5);
   imed3=(int)((double)(m)*(0.5+gauss2)+0.5);

   mm=n/mstep;
   mm=mm*mstep;


   for (i=1; i<=mm; i=i+mstep) {

	 for (j=0; j<m; j++) {

	     dmag2[j]=dmag[i+j];

	 } /* for j */

	 qqsort(dmag2,1,m);

	 mag0=mag[i+imed2];
	 median=dmag2[imed2];
	 siglow=dmag2[imed1];
	 sighigh=dmag2[imed3];

         fprintf(outfile,"%12.6lf %12.6lf %12.6lf %12.6lf\n",
	         mag0,median,siglow,sighigh);
	
   } /* for i */


   fclose(outfile);
   fclose(infile);


} /* End */
