/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*   plots.c   SS July 92                                    */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include "image.h"


double timetic=(double)(CLOCKS_PER_SEC);


double pi=3.14159265358979323;

int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header;   
   short **image;

   struct image_head header2;   
   short **image2;

   int i,j,k,l,width,height,tempi,j1,i1,max,i2,j2,col,im2size;
   int min;

   double outorigx,outorigy,f,sh,ch,vh,vv,c,amp,cx,cy,bx,by;
   double x,y,z,xs,ys,sv,cv,rot,sw,cw,v,x1,y1,w,im2orig;
   double maxamp,xs2,ys2,s,alfa,fmin,fmax,eps,bg,inorigx,inorigy;
   double fheight,amp2;
   
   clock_t time1,timediff;

   int *colv;


   if (argc<6) {
printf("Usage: %s inimage outimage rotangelr vertangelr ampr powerr\n",argv[0]);
      exit(0);
   } /* if */

  
   ReadHeader(argv[1],&header);

   image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],image);

   header2.id=image_magic;
   header2.typ=im_s16;
   header2.off=512;
   header2.hgh=(int)(sqrt(2.0)*header.hgh)+1;
   header2.wid=(int)(sqrt(2.0)*header.wid)+1;
   
   image2 = (short **)calloc(header2.hgh,sizeof(short*));

   CreateIm_s16(image2,header2.hgh,header2.wid);



v=atof(argv[3]);
vv=atof(argv[4]);
amp2=atof(argv[5]);
alfa=atof(argv[6]);

if (amp2 > 1.0) {
   amp2=1.0;
} /* if */


 colv=(int *)calloc(32768+1,sizeof(int));


 for (i=0; i<32768; i++) {
    x=(double)(i);
    colv[i]=(int)(5000.0*sin(2.0*pi*x/(x/20.0+500.0))+20000.0);
 } /* for i */

 vh=0.0*pi/180.0;
 vv=vv*pi/180.0;
 v=v*pi/180;

 width=header.wid;
 height=header.hgh;
 fheight=(double)(height);

 im2size=header2.hgh;
 im2orig=(double)(im2size-1)/2.0;

 inorigx=(double)(width-1)/2.0;
 inorigy=(double)(height-1)/2.0;

 outorigx=inorigx;
 outorigy=inorigy*(1.0-0.4*cos(vv));


 cx=1.0;
 cy=1.0;
 bx=0.8;
 by=0.8;

 f=6.0*320.0*by*fheight/512.0;
 maxamp=330.0*by*fheight/512.0;

 eps=1.0e-6;

for (i=0; i<im2size; i++) {
   for (j=0; j<im2size; j++) {
      image2[j][i]=0;
   } /* for j */
} /* for i */

 sv=sin(v);
 cv=cos(v);

 min=32768;
 max=0;

 for (i=0; i<width; i++) {
    for (j=0; j<height; j++) {
       x=(double)(i)-inorigx;
       y=(double)(j)-inorigy;

       x1=x*cv-y*sv;
       y1=x*sv+y*cv;

       i1=(int)(x1+im2orig+0.5);
       j1=(int)(y1+im2orig+0.5);

       tempi=image[j][i]+1;
       if (tempi > max) {
          max=tempi;
       }
       if (tempi < min) {
          min=tempi;
       }
       image2[j1][i1]=tempi;
    } /* for j */
 } /* for i */

for (i=0; i<width; i++) {
   for (j=0; j<height; j++) {
      image[j][i]=32767;
   } /* for j */
} /* for i */

sh=sin(vh); 
ch=cos(vh);
sv=sin(vv); 
cv=cos(vv);

fmax=(double)(max);
fmin=(double)(min);
bg=exp(alfa*log(1.0/(fmax-fmin+1.0)));
amp=maxamp/(1.0-bg);

time1=clock();

for (j=0; j<im2size; j++) {
   for (i=0; i<im2size; i++) {
   if (image2[j][i] > 0) {
      x=bx*((double)(i)-im2orig);
      y=-by*((double)(j)-im2orig);
      z=(double)(image2[j][i]);
      z=amp2*amp*(exp(alfa*log((z-fmin+1.0)/(fmax-fmin+1.0)))-bg);
      s=z-(double)((int)(z))-eps;
      while (s < z)  {
         xs=cx*(x*(f-y*cv)/f*ch+y*sh)+outorigx;
         ys=cy*(s*(f-y*ch)/f*cv+y*sv)+outorigy;
         i1=(int)(xs+0.5);
         j1=(int)(fheight-ys+0.5);
         col=colv[(int)(32767.0/maxamp*s)];
         if ((i1>=0) && (i1<width)) {
            if ((j1>=0) && (j1<height)) {
                image[j1][i1]=col;
            } /* if */
         } /* if */
         s=s+1.0;
      } /* while */
   } /* if */
   } /* for j */
} /* for i */ 

timediff=clock()-time1;

printf("%12.2f\n",(double)(timediff)/timetic);

WriteImage(argv[2],image,header,im_s16);

} /* End  */
