/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  slice.c   SS July 92, Dec 97                             */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>


int main(argc,argv) 
int argc;
char *argv[];
{
  
   FILE *infile;

   int i,j,xi,yi;
   int xsize,ysize;

   int xys[3];

   double x1,y1,x2,y2,q,qx,qy,le,x,y;

   float fff[3];

   float **image;
   float *mpimage;


   if (argc<5) {
      printf("Usage: %s image.fff x1r y1r x2r y2r\n",argv[0]);
      exit(0);
   } /* if */

  
   if ((infile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */


   x1=atof(argv[2]);
   y1=atof(argv[3]);
   x2=atof(argv[4]);
   y2=atof(argv[5]);

   fread(&xys[1],sizeof(int),2,infile);

   xsize=xys[1];
   ysize=xys[2];

   image=(float **)calloc(ysize,sizeof(float*));
   mpimage=(float *)calloc(xsize*ysize,sizeof(float));

   for (i=0; i<ysize; i++) {
      image[i]=mpimage;
      mpimage=mpimage+xsize;
   } /* for i */

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
          fread(&fff[1],sizeof(float),1,infile);
	  image[i][j]=fff[1];
      } /* for j */
   } /* for i */


   q=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));

   qx=(x2-x1)/q;
   qy=(y2-y1)/q;

   x=x1;
   y=y1;
   le=0.0;

   while (le <= q) {

      xi=(int)(x+0.5);
      yi=(int)(y+0.5);

      printf("%5.1f %17.5f\n",le,(double)(image[yi][xi]));

      x=x+qx;
      y=y+qy;

      le=le+1.0;

   } /* while */

   fclose(infile);
   
} /* End  */
